package com.doc.chat.service.impl;

import com.alibaba.fastjson2.JSON;
import com.doc.biz.service.IDocInfoService;
import com.doc.chat.domain.ChatMsg;
import com.doc.chat.mapper.ChatMsgMapper;
import com.doc.chat.service.IChatMsgService;
import com.doc.common.constant.CacheConstants;
import com.doc.common.core.domain.entity.SysUser;
import com.doc.common.utils.DateUtils;
import com.doc.common.utils.SecurityUtils;
import com.doc.common.utils.StringUtils;
import com.doc.system.service.ISysUserService;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 聊天消息管理Service业务层处理
 *
 * @author wukai
 * @date 2023-08-24
 */
@Service
public class ChatMsgServiceImpl implements IChatMsgService {
    @Resource
    private ChatMsgMapper chatMsgMapper;
    @Resource
    private ISysUserService userService;
    @Resource
    private StringRedisTemplate stringRedisTemplate;
    @Resource
    private IDocInfoService docInfoService;


    /**
     * 查询聊天消息管理
     *
     * @param msgId 聊天消息管理主键
     * @return 聊天消息管理
     */
    @Override
    public ChatMsg selectChatMsgByMsgId(Long msgId) {
        return chatMsgMapper.selectChatMsgByMsgId(msgId);
    }

    /**
     * 查询聊天消息管理列表
     *
     * @param chatMsg 聊天消息管理
     * @return 聊天消息管理
     */
    @Override
    public List<ChatMsg> selectChatMsgList(ChatMsg chatMsg) {
        return chatMsgMapper.selectChatMsgList(chatMsg);
    }

    /**
     * 新增聊天消息管理
     *
     * @param chatMsg 聊天消息管理
     * @return 结果
     */
    @Override
    public int insertChatMsg(ChatMsg chatMsg) {
        String idIndex = getIdIndex(chatMsg.getFromId(), chatMsg.getToId());
        chatMsg.setIdIndex(idIndex);
        chatMsg.setCreateTime(DateUtils.getNowDate());
        return chatMsgMapper.insertChatMsg(chatMsg);
    }

    private String getIdIndex(Long fromId, Long toId) {
        String idIndex = fromId + "" + toId;
        if (fromId > toId) {
            idIndex = toId + "" + fromId;
        }
        return idIndex;
    }

    /**
     * 修改聊天消息管理
     *
     * @param chatMsg 聊天消息管理
     * @return 结果
     */
    @Override
    public int updateChatMsg(ChatMsg chatMsg) {
        return chatMsgMapper.updateChatMsg(chatMsg);
    }

    /**
     * 批量删除聊天消息管理
     *
     * @param msgIds 需要删除的聊天消息管理主键
     * @return 结果
     */
    @Override
    public int deleteChatMsgByMsgIds(Long[] msgIds) {
        return chatMsgMapper.deleteChatMsgByMsgIds(msgIds);
    }

    /**
     * 删除聊天消息管理信息
     *
     * @param msgId 聊天消息管理主键
     * @return 结果
     */
    @Override
    public int deleteChatMsgByMsgId(Long msgId) {
        return chatMsgMapper.deleteChatMsgByMsgId(msgId);
    }

    /**
     * 获取聊天记录
     *
     * @param fromId 当前用户ID
     * @param toId   接收用户ID
     * @return 聊天记录列表
     */
    @Override
    public List<ChatMsg> selectRecordList(Long fromId, Long toId) {
        List<ChatMsg> msgList = chatMsgMapper.selectRecordList(getIdIndex(fromId, toId));
        process(msgList);
        return msgList;
    }

    /**
     * 查询好友列表
     *
     * @param userId 当前用户ID
     * @return 好友列表
     */
    @Override
    public List<ChatMsg> selectFriendList(Long userId) {
        List<ChatMsg> msgList = chatMsgMapper.selectFriendList(userId);
        process(msgList);
        return msgList;
    }

    /**
     * 发送消息
     *
     * @param msg 消息对象
     */
    @Override
    public int send(ChatMsg msg) {
        msg.setFromId(SecurityUtils.getUserId());
        if (msg.getFromId().equals(msg.getToId())) {
            return 0;
        }
        msg.setCreateTime(new Date());
        stringRedisTemplate.convertAndSend(CacheConstants.CHANNEL, JSON.toJSONString(msg));
        return insertChatMsg(msg);
    }

    /**
     * 设置用户及文件信息
     *
     * @param msgList 消息列表
     */
    private void process(List<ChatMsg> msgList) {
        Map<Long, SysUser> userMap = userService.selectAllUser();
        msgList.forEach(msg -> {
            String type = "1";
            if (type.equals(msg.getMsgType())) {
                if (StringUtils.isNotEmpty(msg.getContent())) {
                    Long docId = new Long(msg.getContent());
                    msg.setFile(docInfoService.selectDocInfoByDocId(docId));
                }
            }
            SysUser fromUser = userMap.get(msg.getFromId());
            SysUser toUser = userMap.get(msg.getToId());
            if (fromUser != null) {
                msg.setFromName(fromUser.getNickName());
                msg.setFromAvatar(fromUser.getAvatar());
            }
            if (toUser != null) {
                msg.setToName(toUser.getNickName());
                msg.setToAvatar(toUser.getAvatar());
            }
        });
    }

}
